from gnutools.utils import listfiles
import os
from concurrent.futures import ProcessPoolExecutor, as_completed
from tqdm import tqdm
import numpy as np

def files_exist(files, dst_root):
    return [file for file in files if os.path.exists(os.path.join(dst_root, file))]

class FileSync:
    def __init__(self, src_root, dst_root):
        self.src_root=src_root
        self.dst_root=dst_root
        self.d_src, self.d_dst = {}, {}

    def compare_file(self, file):
        def compare(v1, v2, k):
            if k == "st_size":
                return v1 == v2
            if k == "st_atime":
                return v1 < v2

        return (file, dict([(k, compare(v, self.d_dst[file][k], k)) for k,v in self.d_src[file].items()]))

    def run(self):
        def stat(file):
            stats = os.stat(file)
            return {
                "st_atime": stats.st_atime,
                "st_size": stats.st_size
            }

        files_src = [file.replace(self.src_root, "") for file in listfiles(self.src_root)]
        with ProcessPoolExecutor(1) as e:
            N = len(files_src)
            batch = N//24
            fs = [e.submit(files_exist, files_src[k_start:k_start + batch], self.dst_root)
                  for k_start in np.arange(0, N, batch)]
            files_dst = []
            for f in tqdm(as_completed(fs)):
                files_dst.extend(f.result())

        # files_dst = [file for file in files_src if os.path.exists(os.path.join(self.dst_root, file))]
        #
        self.doublons = set(files_src).intersection(files_dst)
        # files_dst = listfiles(self.dst_root)
        # self.d_src = dict([(os.path.join(self.src_root, file), stat(os.path.join(self.src_root, file)))
        #                    for file in files_src])
        # self.d_src = dict([(os.path.join(self.dst_root, file), stat(os.path.join(self.src_root, file)))
        #                    for file in files_dst])
        #
        #
        # inter_files = set(self.d_src.keys()).intersection(set(self.d_dst.keys()))
        # # self.doublons = dict([self.compare_file(file) for file in inter_files])
        # a=1
    def move(self, or_conds={"st_size":False, "st_atime":False}):
        def in_condition(stats):
            if or_conds["st_size"] == stats["st_size"]:
                return True
            if or_conds["st_atime"] == stats["st_atime"]:
                return True

        move_files = [file for file, stats in self.doublons.items() if in_condition(stats)]

        # print(move_files)

        for file in move_files:
            self.move_file(os.path.join(self.src_root, file), os.path.join(self.dst_root, file))

    @staticmethod
    def move_file(src_file, dst_file):
        command = "mv {src_file} {dst_file}".format(src_file=src_file, dst_file=dst_file)
        print(command)

if __name__=="__main__":
    import time

    fsync = FileSync("/mnt/9DWNas-02/MITSUI-03/", "/mnt/9DWNas-02/MITSUI-01/")
    t0 = time.time()
    fsync.run()
    print(time.time() - t0)
    # fsync.move(or_conds={"st_size":True, "st_atime":True})